/* A Bison parser, made by GNU Bison 1.875.  */

/* Skeleton parser for Yacc-like parsing with Bison,
   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

/* As a special exception, when this file is copied by Bison into a
   Bison output file, you may use that output file without restriction.
   This special exception was added by the Free Software Foundation
   in version 1.24 of Bison.  */

/* Tokens.  */
#ifndef YYTOKENTYPE
# define YYTOKENTYPE
   /* Put the tokens into the symbol table, so that GDB and other debuggers
      know about them.  */
   enum yytokentype {
     YSHAPES = 258,
     YELASTIC = 259,
     YPADDING = 260,
     YSAMPLE = 261,
     YENDSAMPLE = 262,
     YBOX = 263,
     YEND = 264,
     YUNREC = 265,
     YREPLACE = 266,
     YREVERSE = 267,
     YTO = 268,
     YWITH = 269,
     YCHGDEL = 270,
     KEYWORD = 271,
     WORD = 272,
     STRING = 273,
     SHAPE = 274,
     YNUMBER = 275,
     YRXPFLAG = 276,
     YDELWORD = 277
   };
#endif
#define YSHAPES 258
#define YELASTIC 259
#define YPADDING 260
#define YSAMPLE 261
#define YENDSAMPLE 262
#define YBOX 263
#define YEND 264
#define YUNREC 265
#define YREPLACE 266
#define YREVERSE 267
#define YTO 268
#define YWITH 269
#define YCHGDEL 270
#define KEYWORD 271
#define WORD 272
#define STRING 273
#define SHAPE 274
#define YNUMBER 275
#define YRXPFLAG 276
#define YDELWORD 277




#if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
#line 416 "parser.y"
typedef union YYSTYPE {
    char *s;
    char c;
    shape_t shape;
    sentry_t sentry;
    int num;
} YYSTYPE;
/* Line 1249 of yacc.c.  */
#line 88 "parser.h"
# define yystype YYSTYPE /* obsolescent; will be withdrawn */
# define YYSTYPE_IS_DECLARED 1
# define YYSTYPE_IS_TRIVIAL 1
#endif

extern YYSTYPE yylval;



